import click as ck
from daily_tasks.commands.main_commands import add_task, view_tasks
from daily_tasks.commands.modification_commands import modify_description, modify_priority, modify_due_date, modify_status
from daily_tasks.commands.removal_commands import delete_done_tasks, delete_task
from daily_tasks.commands.filter_commands import filter_tasks_by_priority, filter_tasks_by_due_date, filter_tasks_by_status


@ck.group
def daily_tasks() -> None:
    """A tasks manager for those who like work from shell."""


daily_tasks.add_command(add_task)
daily_tasks.add_command(view_tasks)
daily_tasks.add_command(modify_description)
daily_tasks.add_command(modify_priority)
daily_tasks.add_command(modify_due_date)
daily_tasks.add_command(modify_status)
daily_tasks.add_command(delete_done_tasks)
daily_tasks.add_command(delete_task)
daily_tasks.add_command(filter_tasks_by_priority)
daily_tasks.add_command(filter_tasks_by_due_date)
daily_tasks.add_command(filter_tasks_by_status)



from click.testing import CliRunner

from daily_tasks.commands.main_commands import view_tasks, add_task, create_tasks_file

# Delete this test
def test_create_tasks_file():
    runner = CliRunner()
    test_task_data_file = 'Tasks_test.json'
    result = runner.invoke(create_tasks_file, ['--file_name', test_task_data_file])
    assert result.exit_code == 0, f"Command failed: {result.exception}\n{result.output}"

def test_add_task():
    runner = CliRunner()

    description = "Complete unit test"
    priority = "H"
    due_date = "2024/03/15"
    status = "To-do"
    test_task_data_file = "Tasks_test.json" # Change to TASKS_FILE_NAME constant.

    result = runner.invoke(add_task, [
        '--description', description,
        '--priority', priority,
        '--due-date', due_date,
        '--status', status,
        '--file_name', test_task_data_file,
    ])

    assert result.exit_code == 0, f"Command failed: {result.exception}\n{result.output}"

def test_view_tasks():
    runner = CliRunner()
    test_task_data_file = 'Tasks_test.json' # Change to TASKS_FILE_NAME constant.
    result = runner.invoke(view_tasks, ['--file_name', test_task_data_file])

    assert result.exit_code == 0, f"Command failed: {result.exception}\n{result.output}"

test_create_tasks_file()
test_add_task()
test_view_tasks()



"""Utilities"""
from datetime import datetime
from json import dump
import click as ck


DUE_DATE_FORMAT = ['%Y/%m/%d']
PRIORITIES = ['H', 'M', 'L', ' ']
STATUS = ['To-do', 'In-progress', 'Done', ' ']
TASKS_FILE_NAME = 'tasks.json'

def get_due_date_default_value() -> str:
    """Get default value for due_date parameter"""
    today_datetime_obj = datetime.today()
    today_date_obj = today_datetime_obj.date()
    today_date_obj_formatted = today_date_obj.strftime(DUE_DATE_FORMAT[0])
    today_date_str_formatted = str(today_date_obj_formatted)

    return today_date_str_formatted

def stylized_tasks_printing(task_id, description, priority, due_date, status) -> None:
    if priority == PRIORITIES[0]:
        if status == STATUS[0]:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='red', bold=True),
                ck.style(text=status, fg='red'),
                ck.style(text=due_date, bold=True)
            ))
        elif status == STATUS[1]:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='red', bold=True),
                ck.style(text=status, fg='yellow'),
                ck.style(text=due_date, bold=True)
            ))
        else:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='red', bold=True),
                ck.style(text=status, fg='green'),
                ck.style(text=due_date, bold=True)
            ))
    elif priority == PRIORITIES[1]:
        if status == STATUS[0]:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='yellow', bold=True),
                ck.style(text=status, fg='red'),
                ck.style(text=due_date, bold=True)
            ))
        elif status == STATUS[1]:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='yellow', bold=True),
                ck.style(text=status, fg='yellow'),
                ck.style(text=due_date, bold=True)
            ))
        else:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='yellow', bold=True),
                ck.style(text=status, fg='green'),
                ck.style(text=due_date, bold=True)
            ))
    else:
        if status == STATUS[0]:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='white', bold=True),
                ck.style(text=status, fg='red'),
                ck.style(text=due_date, bold=True)
            ))
        elif status == STATUS[1]:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='white', bold=True),
                ck.style(text=status, fg='yellow'),
                ck.style(text=due_date, bold=True)
            ))
        else:
            ck.echo('{}. {} - {} - {} - {}'.format(
                ck.style(text=task_id, fg='black'),
                description,
                ck.style(text=priority, fg='white', bold=True),
                ck.style(text=status, fg='green'),
                ck.style(text=due_date, bold=True)
            ))

def create_tasks_file(file_name) -> None:
    with open(file_name, 'w', encoding='utf-8') as tasks_file:
        dump([], tasks_file)



import json
import click as ck
from daily_tasks.commands import utilities


@ck.command
def delete_done_tasks():
    """Delete all your done tasks."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as reading_tasks_file:
        tasks = json.load(reading_tasks_file)

    done_status = utilities.STATUS[2]

    for task in tasks:
        if task['status'] == done_status:
            task_index = tasks.index(task)
            tasks.pop(task_index)
    
    with open(utilities.TASKS_FILE_NAME, 'w', encoding='utf-8') as writing_tasks_file:
        json.dump(tasks, writing_tasks_file, indent=2)


@ck.command
@ck.option('-id', '--task-id',
           required=True,
           type=ck.INT)
def delete_task(task_id):
    """Delete a task."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as reading_tasks_file:
        tasks = json.load(reading_tasks_file)

    for task in tasks:
        if task['id'] == task_id:
            task_index = tasks.index(task)
            tasks.pop(task_index)
            ck.echo(ck.style('Task deleted successfully.',
                             bold=True))

    with open(utilities.TASKS_FILE_NAME, 'w', encoding='utf-8') as writing_tasks_file:
        json.dump(tasks, writing_tasks_file, indent=2)




from datetime import datetime # pylint: disable=unused-import
import json
import click as ck
from daily_tasks.commands import utilities


@ck.command
@ck.option('-p', '--priority',
           required=True,
           type=ck.Choice(utilities.PRIORITIES, case_sensitive=False)
           )
def filter_tasks_by_priority(priority):
    """Filter your tasks by priority."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as reading_tasks_file:
        tasks = json.load(reading_tasks_file)

    priority_upper = priority.upper()

    for task in tasks:
        if task['priority'] == priority_upper:

            task_id = task['id']
            description = task['description']
            priority = task['priority']
            due_date = task['due_date']
            status = task['status']

            utilities.stylized_tasks_printing(task_id, description, priority, due_date, status)


@ck.command
@ck.option('-dd', '--due-date',
           required=True,
           type=ck.DateTime(formats=utilities.DUE_DATE_FORMAT)
           )
def filter_tasks_by_due_date(due_date):
    """Filter your tasks by due date."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as reading_tasks_file:
        tasks = json.load(reading_tasks_file)

    due_date_date_object = due_date.date()
    due_date_formatted = due_date_date_object.strftime(utilities.DUE_DATE_FORMAT[0])

    for task in tasks:
        if task['due_date'] == due_date_formatted:

            task_id = task['id']
            description = task['description']
            priority = task['priority']
            due_date = task['due_date']
            status = task['status']

            utilities.stylized_tasks_printing(task_id, description, priority, due_date, status)


@ck.command
@ck.option('-s', '--status',
           required=True,
           type=ck.Choice(utilities.STATUS, case_sensitive=False))
def filter_tasks_by_status(status):
    """Filter your tasks by status."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as reading_tasks_file:
        tasks = json.load(reading_tasks_file)

    status_capitalize = status.capitalize()

    for task in tasks:
        if task['status'] == status_capitalize:

            task_id = task['id']
            description = task['description']
            priority = task['priority']
            due_date = task['due_date']
            status = task['status']

            utilities.stylized_tasks_printing(task_id, description, priority, due_date, status)



from daily_tasks.commands import utilities
from datetime import datetime # pylint: disable=unused-import
import json
import click as ck


@ck.command
@ck.option('-id', '--task-id',
           required=True,
           type=ck.INT
           )
@ck.option('-d', '--new-description',
           required=True,
           type=ck.STRING
           )
def modify_description(task_id, new_description) -> None:
    """Modify description of a task."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as tasks_file_read:
        tasks = json.load(tasks_file_read)

    for task in tasks:
        if task['id'] == task_id:
            extracted_task = task
            break

    task_index = tasks.index(extracted_task)

    extracted_task['description'] = new_description.capitalize()

    tasks.pop(task_index)
    tasks.insert(task_index, extracted_task)

    with open(utilities.TASKS_FILE_NAME, 'w', encoding='utf-8') as tasks_file_write:
        json.dump(tasks, tasks_file_write, indent=2)


@ck.command
@ck.option('-id', '--task-id',
           required=True,
           type=ck.INT
           )
@ck.option('-p', '--new-priority',
           required=True,
           type=ck.Choice(utilities.PRIORITIES, case_sensitive=False)
           )
def modify_priority(task_id, new_priority):
    """Modify priority of a task."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as tasks_file_read:
        tasks = json.load(tasks_file_read)

    for task in tasks:
        if task['id'] == task_id:
            extracted_task = task
            break

    task_index = tasks.index(extracted_task)

    new_priority_upper = new_priority.upper()
    extracted_task['priority'] = new_priority_upper

    tasks.pop(task_index)
    tasks.insert(task_index, extracted_task)

    with open(utilities.TASKS_FILE_NAME, 'w', encoding='utf-8') as tasks_file_write:
        json.dump(tasks, tasks_file_write, indent=2)


@ck.command
@ck.option('-id', '--task-id',
           required=True,
           type=ck.INT
           )
@ck.option('-dd', '--new-due-date',
           required=True,
           type=ck.DateTime(formats=utilities.DUE_DATE_FORMAT)
           )
def modify_due_date(task_id, new_due_date):
    """Modify due date of a task."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as tasks_file_read:
        tasks = json.load(tasks_file_read)

    for task in tasks:
        if task['id'] == task_id:
            extracted_task = task
            break

    task_index = tasks.index(extracted_task)

    new_due_date_date_object = new_due_date.date()
    new_due_date_formatted = new_due_date_date_object.strftime(utilities.DUE_DATE_FORMAT[0])

    extracted_task['due_date'] = new_due_date_formatted

    tasks.pop(task_index)
    tasks.insert(task_index, extracted_task)

    with open(utilities.TASKS_FILE_NAME, 'w', encoding='utf-8') as tasks_file_write:
        json.dump(tasks, tasks_file_write, indent=2)


@ck.command
@ck.option('-id', '--task-id',
           required=True,
           type=ck.INT
           )
@ck.option('-s', '--new-status',
           required=True,
           type=ck.Choice(choices=utilities.STATUS, case_sensitive=False)
           )
def modify_status(task_id, new_status):
    """Modify status of a task."""
    with open(utilities.TASKS_FILE_NAME, 'r', encoding='utf-8') as tasks_file_read:
        tasks = json.load(tasks_file_read)

    for task in tasks:
        if task['id'] == task_id:
            extracted_task = task
            break

    task_index = tasks.index(extracted_task)

    new_status_capitalize = new_status.capitalize()
    extracted_task['status'] = new_status_capitalize

    tasks.pop(task_index)
    tasks.insert(task_index, extracted_task)

    with open(utilities.TASKS_FILE_NAME, 'w', encoding='utf-8') as tasks_file_write:
        json.dump(tasks, tasks_file_write, indent=2)
